@@ -1,5 +1,6 @@
 import unittest
 from pydawn import utils, webgpu
+import os
 
 class TestUtils(unittest.TestCase):
     def setUp(self):
@@ -22,7 +23,16 @@ class TestUtils(unittest.TestCase):
         test_src = [1, 2, 3, 4]
         utils.write_buffer(self.device, buf, 0, test_src)
         out = utils.read_buffer(self.device, buf)
-        self.assertEqual(list(out), test_src, f"buffer mismatch: (actual) {test_src} != (expected) {out}")
+        self.assertEqual(list(out), test_src, f"buffer mismatch: (actual) {test_src} != (expected) {list(out)}")
+
+    def test_copy_buffer_to_buffer(self):
+        src = utils.create_buffer(self.device, 4, webgpu.WGPUBufferUsage_Storage | webgpu.WGPUBufferUsage_CopySrc | webgpu.WGPUBufferUsage_CopyDst)
+        dst = utils.create_buffer(self.device, 4, webgpu.WGPUBufferUsage_Storage | webgpu.WGPUBufferUsage_CopySrc | webgpu.WGPUBufferUsage_CopyDst)
+        test_src = [4, 8, 16, 32]
+        utils.write_buffer(self.device, src, 0, test_src)
+        utils.copy_buffer_to_buffer(self.device, src, 0, dst, 0, 4)
+        dst_out = utils.read_buffer(self.device, dst)
+        self.assertEqual(list(dst_out), test_src, f"buffer mismatch: (actual) {test_src} != (expected) {list(dst_out)}")
 
     def test_create_shader_module(self):
         shader_source = """
@@ -60,9 +70,13 @@ class TestUtils(unittest.TestCase):
 
     def test_request_adapter_error(self):
         with self.assertRaises(RuntimeError) as ctx:
-            utils.request_adapter_sync(webgpu.WGPUPowerPreference_HighPerformance, webgpu.WGPUBackendType_Vulkan)
+            os.environ['BACKEND_TYPE'] = "D3D11"
+            try:
+                utils.request_adapter_sync(webgpu.WGPUPowerPreference_HighPerformance)
+            finally:
+                del os.environ['BACKEND_TYPE']
 
-        self.assertIn("No supported adapters", str(ctx.exception))
+        self.assertIn("Unsupported backend", str(ctx.exception))
 
     def test_request_device_error(self):
         with self.assertRaises(RuntimeError) as ctx:
